/**
 * Copyright (c) 2016 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#include "buttons_m.h"
#include <stdbool.h>
#include "sdk_errors.h"
#include "bsp.h"
#include "ble_m.h"
#include "nfc_ble_pair_lib.h"

#define NRF_LOG_MODULE_NAME BUTTONS_M
#include "nrf_log.h"
NRF_LOG_MODULE_REGISTER();

#define APP_BTN_EVT_DISCONNECT  BSP_EVENT_KEY_0 /**< Event used to terminate connection with BLE cental device. */
#define APP_BTN_EVT_LESC_OOB    BSP_EVENT_KEY_1 /**< Event used to change pairing mode to LESC OOB pairing. */
#define APP_BTN_EVT_OOB_PAIRING BSP_EVENT_KEY_2 /**< Event used to change pairing mode to Legacy OOB pairing. */
#define APP_BTN_EVT_JUST_WORKS  BSP_EVENT_KEY_3 /**< Event used to change pairing mode to Legacy Just Works pairing. */
#define APP_BTN_ID_BOND_DELETE  1               /**< ID of button used to delete all bonding information. */

/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated by button press.
 */
static void bsp_event_handler(bsp_event_t event)
{
    ret_code_t err_code = NRF_SUCCESS;

    switch (event)
    {
        case APP_BTN_EVT_DISCONNECT:
            NRF_LOG_INFO("Terminating BLE connection.");
            ble_disconnect();
            break;

        case APP_BTN_EVT_JUST_WORKS:
            NRF_LOG_INFO("LEGACY JUST WORKS MODE!");
            err_code = nfc_ble_pair_mode_set(NFC_PAIRING_MODE_JUST_WORKS);
            break;

        case APP_BTN_EVT_OOB_PAIRING:
            NRF_LOG_INFO("LEGACY OOB MODE!");
            err_code = nfc_ble_pair_mode_set(NFC_PAIRING_MODE_OOB);
            break;

        case APP_BTN_EVT_LESC_OOB:
            NRF_LOG_INFO("LESC OOB MODE!");
            err_code = nfc_ble_pair_mode_set(NFC_PAIRING_MODE_LESC_OOB);
            break;

        default:
            break;
    }

    APP_ERROR_CHECK(err_code);
}

bool buttons_init(void)
{
    ret_code_t err_code;

    // Initialize buttons and add bsp buttons task to the application timer.
    err_code = bsp_init(BSP_INIT_BUTTONS | BSP_INIT_LED, bsp_event_handler);
    APP_ERROR_CHECK(err_code);

    return bsp_button_is_pressed(APP_BTN_ID_BOND_DELETE);
}
